import java.util.Set;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.Tokenization;
import com.aliasi.util.ObjectToCounterMap;
import com.aliasi.chunk.Chunk;

public class AdHocPhraseFilter implements PhraseFilter {

    Set<String> mStopPhrases;
    ObjectToCounterMap<String> mTokenNgramCounts;
    ObjectToCounterMap<String> mTokenNgramDocCounts;
    TokenizerFactory mTokFactory;
    int mCounter = 0;
    
    public AdHocPhraseFilter(Set<String> stopPhrases,
			     ObjectToCounterMap<String> tokenNgramCounts,
			     ObjectToCounterMap<String> tokenNgramDocCounts,
			     TokenizerFactory tokFactory) {
	mStopPhrases = stopPhrases;
	mTokenNgramCounts = tokenNgramCounts;
	mTokenNgramDocCounts = tokenNgramDocCounts;
	mTokFactory = tokFactory;
    }

    public boolean failPhrase(Chunk npChunk, String phrase) {
	
	if (mStopPhrases.contains(phrase)) {
	    ++mCounter;
	    System.out.println("Stop Phrase list blocked: " + phrase + " " + mCounter);
	    return true;
	    
	}
	if (npChunk.type() != "noun") {
	    return true;
	}
	String normalizedPhrase = normalize(mTokFactory,phrase);
	//System.out.println("|" + normalizedPhrase + "|");
	if (mTokenNgramCounts.getCount(normalizedPhrase) > 5 ){
	    ++mCounter;
	    System.out.println(mCounter + " Skipping " + phrase + ": word count=" 
			       + mTokenNgramCounts.getCount(normalizedPhrase)
			       + " doc count=" + mTokenNgramDocCounts.getCount(normalizedPhrase));
	    return true;
	}
	
	if (phrase.matches("\\d+")){  
	    return true;
	}
    
	return false;

    }

    static String normalize(TokenizerFactory tokFactory, String phrase) {
	StringBuilder sb = new StringBuilder();
	Tokenization tok = new Tokenization(phrase,tokFactory);
	String[] tokens = tok.tokens();
	for (int i = 0; i < tokens.length; ++i) {
	    sb.append(tokens[i]);
	    if (i != tokens.length - 1) {
		sb.append(" ");
	    }
	}
	return sb.toString();
    }

}