import com.aliasi.util.Strings;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.ModifyTokenTokenizerFactory;

import java.io.File;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;


import java.util.HashMap;
import java.util.Map;
import java.util.zip.GZIPInputStream;


public class CommonWordFilterTokenizerFactory extends ModifyTokenTokenizerFactory {

    Map<String,Integer> mPhraseCountMap;
    int mCommonWordCutoff;
    

    public CommonWordFilterTokenizerFactory(TokenizerFactory factory, File phraseCountsFile, int cutoff) throws IOException {
	super(factory);
	mCommonWordCutoff = cutoff;
	loadInput(phraseCountsFile);
    }

    public String modifyToken(String token) {
	Integer count = mPhraseCountMap.get(token);
	if (count != null && count > mCommonWordCutoff) {
	    return null;
	}
	else {
	    return token;
	}
    }

    void loadInput(File phraseCountFile) throws FileNotFoundException, IOException, UnsupportedEncodingException {

	GZIPInputStream zipIn 
	    = new GZIPInputStream(new FileInputStream(phraseCountFile));
	InputStreamReader reader = new InputStreamReader(zipIn,Strings.UTF8);
	BufferedReader bufReader = new BufferedReader(reader);
	Map<String,Integer> phraseCountMap = new HashMap<String,Integer>();
	String line = null;
	while ((line = bufReader.readLine()) != null) {
	    int index = line.indexOf(' ');
	    if (index < 0) continue; // allows empty lines
	    String countString = line.substring(0,index);
	    String phrase = line.substring(index+1);
	    int count = -1;
	    try {
		count = Integer.parseInt(countString);
		phraseCountMap.put(phrase,count);
	    } catch (NumberFormatException e) {
		String msg = "Illegal count substring in n-gram resource."
		    + " resource path=" + phraseCountFile
		    + " line=" + line;
		throw e;
	    }
	}
	mPhraseCountMap = phraseCountMap;
    }



    static final long serialVersionUID = -7949489227829523271L;
}
