import com.aliasi.classify.DynamicLMClassifier;
import com.aliasi.classify.JointClassification;
import com.aliasi.lm.TokenizedLM;
import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.LowerCaseTokenizerFactory;
import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.Tokenizer;

import com.aliasi.util.Files;
import com.aliasi.util.ScoredObject;
import com.aliasi.util.Strings;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.Iterator;

import org.supercsv.io.CsvListWriter;
import org.supercsv.prefs.CsvPreference;



public class DedupeSimple {

    public static void main(String[] args) throws IOException {
	Set<String> seenBefore = new HashSet<String>();
	int rawCount = 0;
	int filteredCount = 0;
	int numFolds = 0; //vistTrain visits entire corpus
	StringBuilder sb = new StringBuilder();
	File dataDir = new File(args[0]);
	File outDir = new File(args[1]);
	String outFile = args[2];
	String keepCategories = args[3];
	FilteredXValCorpus corpus 
	    = new FilteredXValCorpus(numFolds,dataDir,keepCategories);


	TokenizerFactory tokFactory
	    = new RegExTokenizerFactory("\\S+");

	tokFactory
	  = new LowerCaseTokenizerFactory(tokFactory);

	tokFactory
	 = new HttpFilterTokenizerFactory(tokFactory);


	String[] texts = corpus.texts();
	
	File output = new File(outDir,outFile); //TweetWriter.nextFile(outDir,outFile,"");
	System.out.println("Writing to: " + output.toString());
	FileOutputStream stream =  new FileOutputStream(output);
	OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	CsvListWriter writer 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 

	ArrayList<String> row = new ArrayList<String>();
	row.add("Order");
	row.add("");
	row.add("novelty");
	row.add("Tweet");
	writer.write(row);

	int seenCounter = 0;
	for (int i=0; i < texts.length; ++i) {
	    ++rawCount;
	    String tweet = texts[i];
	    String tokenNormalizedTweet = buildStringFromTokens(tokFactory,tweet);
	    int numTokens = countTokens(tokFactory,tweet);
	    String msg;
	    //tokenNormalizedTweet = tweet;
	    if (seenBefore.contains(tokenNormalizedTweet)) {
		msg = "seen: ";
		++seenCounter;
	    }
	    else {
		msg = "notseen: ";
		seenBefore.add(tokenNormalizedTweet);
	    }

	    row.clear();
	    row.add(Integer.toString(rawCount));
	    row.add("");
	    row.add(msg);
	    row.add(tweet);
	    writer.write(row);

	}
	writer.close();
	System.out.println(seenCounter + " Seen of " + rawCount);
    }


    static String buildStringFromTokens(TokenizerFactory factory,
				     String tweet) {
	char[] chars = tweet.toCharArray();
	Tokenizer tokenizer 
	    = factory.tokenizer(tweet.toCharArray(),0,chars.length);
	String token = null;
	StringBuilder sb = new StringBuilder();
	sb.append(tokenizer.nextWhitespace());
	while( (token = tokenizer.nextToken()) != null ) {
	    sb.append(token);
	    sb.append(tokenizer.nextWhitespace());
	}
	return sb.toString();
    }

    static int countTokens(TokenizerFactory factory,
		    CharSequence tweet) {
	int count = 0;
	//System.out.println(tweet);
	char[] tweetChar = tweet.toString().toCharArray(); 
	Tokenizer toks = factory.tokenizer(tweetChar,0,tweetChar.length);
	Iterator<String> it = toks.iterator();
	while(it.hasNext() ) {
	    	    it.next();
	    //  System.out.println(it.next());
	    ++count;
	}
	return count;
    }
}
