import com.aliasi.classify.DynamicLMClassifier;
import com.aliasi.classify.JointClassification;
import com.aliasi.lm.TokenizedLM;
import com.aliasi.spell.JaccardDistance;

import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.LowerCaseTokenizerFactory;
import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.Tokenizer;

import com.aliasi.util.Files;
import com.aliasi.util.ObjectToDoubleMap;
import com.aliasi.util.ScoredObject;
import com.aliasi.util.Strings;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.Iterator;

import org.supercsv.io.CsvListWriter;
import org.supercsv.prefs.CsvPreference;



public class DeduplicateBatch {

    public static void main(String[] args) throws IOException {
	Set<String> seenBefore = new HashSet<String>();
	int rawCount = 0;
	int filteredCount = 0;
	int numFolds = 0; //vistTrain visits entire corpus
	StringBuilder sb = new StringBuilder();
	File dataDir = new File(args[0]);
	File outDir = new File(args[1]);
	String outFile = args[2];
	String categories = args[3];
	FilteredXValCorpus corpus 
	    = new FilteredXValCorpus(numFolds,dataDir,categories);

	TokenizerFactory tokFactory
	    = new RegExTokenizerFactory("\\S+");

	tokFactory
	    = new LowerCaseTokenizerFactory(tokFactory);

	tokFactory
	    = new HttpFilterTokenizerFactory(tokFactory);

	String[] texts = corpus.texts();
	
	File output = new File(outDir,outFile); //TweetWriter.nextFile(outDir,outFile,"csv");
	System.out.println("Writing to: " + output.toString());
	FileOutputStream stream =  new FileOutputStream(output);
	OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	CsvListWriter writer 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 

	ArrayList<String> row = new ArrayList<String>();
	row.add("Order");
	row.add("Token Estimate");
	row.add("novelty");
	row.add("Tweet");
	writer.write(row);
	JaccardDistance jaccardD = new JaccardDistance(tokFactory);
	ObjectToDoubleMap<String> map = new ObjectToDoubleMap<String>();
	for (int i=0; i < texts.length; ++i) {
	    ++rawCount;
	    String tweet = texts[i];
	    for (int j=0; j < texts.length; ++j) {
		String comparisionTweet = texts[j];
		double proximity = jaccardD.proximity(tweet,comparisionTweet);
		//		System.out.println("--------\n" + tweet + "\n" + comparisionTweet + "\n" + proximity);
		if (proximity < 1.0) {
		    //System.out.println("adding");
		    map.set(comparisionTweet,jaccardD.proximity(tweet,comparisionTweet));
		}
	    }
	    List<ScoredObject<String>> list =
		map.scoredObjectsOrderedByValueList();

	    //System.out.println(tweet);
	    //System.out.println(list.size());
	    String closestTweet = null;
	    double distance = 0.0;
	    if (list.size() == 0) {
		closestTweet = "";
		distance = 1.0;
	    }
	    else {
		closestTweet = list.get(0).getObject();
		distance = list.get(0).score();
	    }
	    String msg;
	    if (seenBefore.contains(tweet)) {
		msg = "seen: ";
	    }
	    else {
		msg = "notseen: ";
		seenBefore.add(tweet);
	    }

	    row.clear();
	    row.add(Integer.toString(rawCount));
	    row.add(Double.toString(distance));
	    row.add(msg);
	    row.add(tweet);
	    row.add(closestTweet);
	    writer.write(row);
	    
	}
	writer.close();

	System.out.println("Got " + rawCount + " tweets with " + filteredCount + " filtered");

    }


    static int countTokens(TokenizerFactory factory,
		    CharSequence tweet) {
	int count = 0;
	//System.out.println(tweet);
	char[] tweetChar = tweet.toString().toCharArray(); 
	Tokenizer toks = factory.tokenizer(tweetChar,0,tweetChar.length);
	Iterator<String> it = toks.iterator();
	while(it.hasNext() ) {
	    	    it.next();
	    //  System.out.println(it.next());
	    ++count;
	}
	return count;
    }
}
