
import com.aliasi.spell.JaccardDistance;

import com.aliasi.tokenizer.Tokenizer;
import com.aliasi.tokenizer.TokenizerFactory;

import com.aliasi.util.Files;
import com.aliasi.util.Strings;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.supercsv.io.CsvListReader;
import org.supercsv.io.CsvListWriter;
import org.supercsv.prefs.CsvPreference;

public class ExploreJaccard {

    public static void main(String[] args) throws IOException {
	File dataDir = new File(args[0]);
	List<String> texts = parseFile(dataDir);
	TokenizerFactory tokFactory
	    = new NormalizedTokenizerFactory(); //implements normalization
	File output = new File(args[1]); 
	System.out.println("Writing to: " + output.toString());
	FileOutputStream stream =  new FileOutputStream(output);
	OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	CsvListWriter writer 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 
	processAndWrite(texts,tokFactory,writer);
	writer.close();
    }

    static void processAndWrite(List<String> texts, TokenizerFactory tokFactory, CsvListWriter writer) throws IOException {
	System.out.println("Have " + texts.size() + " tweets");
	ArrayList<String> row = new ArrayList<String>();
	row.add("Order");
	row.add("Proximity Score");
	row.add("Truth");	
	row.add("Tweet1");
	row.add("Tweet2");
	writer.write(row); //initialize the csv file with headers
	JaccardDistance jaccardD = new JaccardDistance(tokFactory);
	int filteredCount = 0;
	List<String> candidateTweets 
	    = filterNormalizedDuplicates(texts, 
					 tokFactory); //throw out easy cases
	System.out.println("Normalized duplicate filter leaves " + candidateTweets.size() + " tweets");
	row = new ArrayList<String>();
	for (int i = 0; i < candidateTweets.size(); ++i) {
	    String closestTweet = "default value";
	    double closestProximity = -1d;
	    String targetTweet = candidateTweets.get(i);
	    for (int j = 0; j < candidateTweets.size(); ++j ) {//cross product, ouchy, ow ow. 
		String comparisionTweet = candidateTweets.get(j);
		double thisProximity 
		    = jaccardD.proximity(targetTweet,comparisionTweet);
		if (i != j) { // can't match self
		    if (closestProximity < thisProximity) {
			closestTweet = comparisionTweet;
			closestProximity = thisProximity;
		    }
		}
	    }
	    row.clear();
	    row.add("" + i);
	    row.add(Double.toString(closestProximity));
	    row.add("");
	    row.add(targetTweet);
	    row.add(closestTweet);
	    writer.write(row);
	}
    }

    static List<String> filterNormalizedDuplicates(List<String> texts, TokenizerFactory tokFactory) {
	List<String> returnList = new ArrayList<String>();
	Set<String> seenBefore = new HashSet<String>();
	for (int i=0; i < texts.size(); ++i) {
	    String rawTweet = texts.get(i);
	    String normalizedTweet 
		= buildStringFromTokensRemoveSeparators(tokFactory,rawTweet);
	    if (!seenBefore.contains(normalizedTweet)) {
		returnList.add(rawTweet); //this is what we will process
		seenBefore.add(normalizedTweet); //keeping track of normalized entries
	    }
	}
	return returnList;
    }

    static String buildStringFromTokensRemoveSeparators(TokenizerFactory factory,
							String tweet) {
	char[] chars = tweet.toCharArray();
	Tokenizer tokenizer 
	    = factory.tokenizer(tweet.toCharArray(),0,chars.length);
	String token = null;
	StringBuilder sb = new StringBuilder();
	while( (token = tokenizer.nextToken()) != null ) {
	    sb.append(token);
	}
	return sb.toString();
    }

    static List<String> parseFile(File file) throws IOException {
	List<String> tweets = new ArrayList<String>();
	FileInputStream fileInput = new FileInputStream(file);
	InputStreamReader fileReader = new InputStreamReader(fileInput,Strings.UTF8);
	CsvListReader reader
	    = new CsvListReader(fileReader, CsvPreference.EXCEL_PREFERENCE);
	try {
	    String[] header = reader.getCSVHeader(true);
	    List<String> csv = null;
	    while ((csv = reader.read()) != null) {
		tweets.add(csv.get(TweetWriter.TWEET_OFFSET));
	    }
	} finally {
	    reader.close();
	}
	return tweets;
    }
    
}