import org.supercsv.io.CsvListReader;
import org.supercsv.prefs.CsvPreference;

import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.classify.ScoredClassification;


import com.aliasi.corpus.ObjectHandler;
import com.aliasi.corpus.XValidatingObjectCorpus;

import com.aliasi.util.Files;
import com.aliasi.util.CommaSeparatedValues;
import com.aliasi.util.Streams;
import com.aliasi.util.Strings;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileInputStream;

import java.io.InputStreamReader;
import java.io.IOException;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.HashSet;
import java.util.Set;


import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.corpus.ObjectHandler;
import java.util.Set;
import com.aliasi.corpus.XValidatingObjectCorpus;

public class FilteredXValCorpus extends XValidatingObjectCorpus<Classified<CharSequence>> {

    Set<String> mKeepClassifications;
    String mEncoding = Strings.Latin1;

    public FilteredXValCorpus(int numFolds, File corpusDir, Set<String> keepers) throws IOException {
	super(numFolds);
	mKeepClassifications = keepers;
	parse(corpusDir);
    }

    public FilteredXValCorpus(int numFolds, File corpusDir, String keepers) throws IOException {
	super(numFolds);
	mKeepClassifications = csvToSet(keepers);
	parse(corpusDir);
    }

    Set<String> csvToSet(String csv) {
	System.out.println(csv);
	String[] categories = csv.split(",");
	Set<String> keepClassifications = new HashSet<String>();
	for (int i = 0; i < categories.length; ++i) {
	    keepClassifications.add(categories[i]);
	}
	return keepClassifications;
    }


    static final long serialVersionUID = -1111L;

    
    public void setKeepers(Set<String> keep) {
	mKeepClassifications = keep;
    }

    public Set<String> categorySet() {
        final Set<String> cats = new HashSet<String>();
        visitCorpus(new ObjectHandler<Classified<CharSequence>>() {
                        public void handle(Classified<CharSequence> c) {
                            cats.add(c.getClassification().bestCategory());
                        }
                    });
        return cats;
    }

    void parse(File file) throws IOException {
        System.out.println("file=" + file);
        if (file.isDirectory()) {
            for (File subFile : file.listFiles())
                parse(subFile);
        } else if (file.getName().endsWith(".csv")) {
            parseFile(file);
	}
    }

    void parseFile(File file) throws IOException {

	FileInputStream fileInput = new FileInputStream(file);
	InputStreamReader fileReader = new InputStreamReader(fileInput,mEncoding);
	
	CsvListReader reader
	    = new CsvListReader(fileReader, CsvPreference.EXCEL_PREFERENCE);
	
	int counter = 2;
	try {
	    
	    String[] header = reader.getCSVHeader(true);
	    System.out.println(counter);
	    List<String> csv = null;
	    while ((csv = reader.read()) != null) {
		//System.out.println(counter);
		//System.out.println(csv);
		String classification = csv.get(TweetWriter.ANNOTATED_CLASS_OFFSET);

		if (mKeepClassifications == null
		    || mKeepClassifications.contains(classification)) {
		    Classification c = new Classification(classification);
		    Classified<CharSequence> classified
			= new Classified<CharSequence>(csv.get(TweetWriter.TWEET_OFFSET),c);
		    //System.out.println(csv.get(TweetWriter.TWEET_OFFSET));
		    handle(classified);
		    ++counter;
		}
	    }
	    
	} finally {
	    reader.close();
	}
    }

    public String[] texts() {
        final List<String> collector = new ArrayList<String>(size());
        visitCorpus(new ObjectHandler<Classified<CharSequence>>() {
		public void handle(Classified<CharSequence> c) {
		    collector.add(c.getObject().toString());
		}
	    });
        return collector.toArray(EMPTY_STRING_ARRAY);
    }
    
    public Set<CharSequence> textsCharSequence() {
        final Set<CharSequence> collector = new HashSet<CharSequence>();
        visitCorpus(new ObjectHandler<Classified<CharSequence>>() {
		public void handle(Classified<CharSequence> c) {
		    collector.add(c.getObject());
		}
	    });
        return collector;
    }
    
    @Override
	
	public String toString() {
	final StringBuilder sb = new StringBuilder();
	sb.append("Size: " + size() + "\n");
	for (String cat : categorySet()) {
	    sb.append("Category: '" + cat + "'\n");
	}
	
	visitCorpus(new ObjectHandler<Classified<CharSequence>>() {
		public void handle(Classified<CharSequence> c) {
		    sb.append(c.getClassification() 
			      + "\t" 
			 + c.getObject().toString() 
			      + "\n");
		}
	    });
	return sb.toString();

    }
    
    
    static final CharSequence[] EMPTY_CHARSEQUENCE_ARRAY = new CharSequence[0];
    static final String[] EMPTY_STRING_ARRAY = new String[0];

    public static void main(String[] args) throws IOException {
	Set<String> keepers = null;

	/*new HashSet<String>();
	keepers.add("b");
	keepers.add("n");
	*/
	FilteredXValCorpus corpus 
	    = new FilteredXValCorpus(10,new File(args[0]),keepers);
	/*	corpus.setFold(0);
	corpus.visitTest(new ObjectHandler<Classified<CharSequence>> () {
		public void handle(Classified<CharSequence> c) {
		    System.out.println(c);
		}
	    });
	*/

	System.out.println("Size: " + corpus.size());
	for (String cat : corpus.categorySet()) {
	    System.out.println("Category: '" + cat + "'");
	}

    }
}

