import com.aliasi.classify.Classified;
import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.lm.TokenizedLM;
import com.aliasi.util.Files;
import com.aliasi.util.ScoredObject;
import com.aliasi.util.AbstractExternalizable;
import com.aliasi.chunk.*;
import com.aliasi.corpus.Corpus;
import com.aliasi.corpus.XValidatingObjectCorpus;
import com.aliasi.corpus.ObjectHandler;
import com.aliasi.hmm.HiddenMarkovModel;
import com.aliasi.hmm.HmmDecoder;
import com.aliasi.sentences.SentenceModel;
import com.aliasi.sentences.IndoEuropeanSentenceModel;
import com.aliasi.sentences.SentenceChunker;


import java.io.File;
import java.io.IOException;

import java.util.SortedSet;




import com.aliasi.util.AbstractCommand;


public class InterestingPhrases extends AbstractCommand {

    public InterestingPhrases(String args[]) {
	super(args);
    }

    public void run() {


	try {
	    File posModelFile 
		= new File("models/pos-en-general-brown.HiddenMarkovModel");
	    HiddenMarkovModel posHmm
		= (HiddenMarkovModel) AbstractExternalizable.readObject(posModelFile);
	    HmmDecoder posTagger= new HmmDecoder(posHmm);
	
	    String categories = getArgument("categories");
	    XValidatingObjectCorpus<Classified<CharSequence>> backgroundCorpus 
		= new FilteredXValCorpus(1,getArgumentFile("background"),categories);

	    XValidatingObjectCorpus<Classified<CharSequence>> foregroundCorpus 
		= new FilteredXValCorpus(0,getArgumentFile("foreground"),categories);
	    
	    System.out.println("Training background model");

	    TokenizedLM backgroundModel = trainLM(backgroundCorpus,3);

	    TokenizedLM foregroundModel = trainLM(foregroundCorpus,3);
	    
	    System.out.println("Done");
	    //backgroundModel.sequenceCounter().prune(3);
	    
	    System.out.println("\nAssembling collocations in Training");


		SortedSet<ScoredObject<String[]>> coll 
		    = backgroundModel.collocationSet(2,
						     2,100);
	    
	    
	    //System.out.println("\nCollocations in Order of Significance:");
	    //report(coll);

	    //	    foregroundModel.sequenceCounter().prune(3);

	    System.out.println("\nAssembling New Terms in Test vs. Training");

	    for (int i = 1; i <=3; ++i) {
		SortedSet<ScoredObject<String[]>> newTerms 
		    = foregroundModel.newTermSet(i,
						 1,
						 100,
						 backgroundModel);
		System.out.println("\nNew Terms in Order of Signficance of length=" + i);
		report(newTerms);
            }    
	    
	}
	catch (Exception e) {
	    throw new RuntimeException(e);
	}
        System.out.println("\nDone.");
    } 


    
      TokenizedLM trainLM(XValidatingObjectCorpus<Classified<CharSequence>> corpus, int nGram, HmmDecoder posTagger) {
	
	TokenizerFactory tokenizerFactory 
	    = IndoEuropeanTokenizerFactory.INSTANCE;

	SentenceModel sentenceModel
	    = new IndoEuropeanSentenceModel(true,false);
	
	final SentenceChunker sentenceChunker
	    = new SentenceChunker(tokenizerFactory,sentenceModel);
	
	final TokenizedLM model =
	    new TokenizedLM(tokenizerFactory,
			    nGram);
	final PhraseChunker npChunker
	    = new PhraseChunker(posTagger,tokenizerFactory);

	corpus.visitCorpus(new ObjectHandler<Classified<CharSequence>> () {
		public void handle(Classified<CharSequence> c) {
		    Chunking chunking = sentenceChunker.chunk(c.getObject());
		    CharSequence text = chunking.charSequence();
		    for (Chunk sentence : chunking.chunkSet()) {
			CharSequence sent = text.subSequence(sentence.start(),sentence.end());
			Chunking npChunking = npChunker.chunk(sent);
			for (Chunk npChunk: npChunking.chunkSet()) {
			    CharSequence np = sent.subSequence(npChunk.start(),npChunk.end());
			    model.train(np);
			}
		    }
		}
	    });
	return model;
      }

    
    TokenizedLM trainLM(XValidatingObjectCorpus<Classified<CharSequence>> corpus, int nGram) {

	TokenizerFactory tokenizerFactory 
	    = IndoEuropeanTokenizerFactory.INSTANCE;
	
	final TokenizedLM model =
	    new TokenizedLM(tokenizerFactory,
			    nGram);
	corpus.visitCorpus(new ObjectHandler<Classified<CharSequence>> () {
		public void handle(Classified<CharSequence> c) {
		    model.train(c.getObject());
		}
	    });
	return model;
    }






    private static void report(SortedSet<ScoredObject<String[]>> nGrams) {
        for (ScoredObject<String[]> nGram : nGrams) {
            double score = nGram.score();
            String[] toks = nGram.getObject();
            report_filter(score,toks);
        }
    }
    
    private static void report_filter(double score, String[] toks) {
        String accum = "";
        for (int j=0; j<toks.length; ++j) {
            if (nonCapWord(toks[j])) return;
            accum += " "+toks[j];
        }
        System.out.println("Score: "+score+" with :"+accum);
    }

    private static boolean nonCapWord(String tok) {
        if (!Character.isUpperCase(tok.charAt(0)))
            return true;
        for (int i = 1; i < tok.length(); ++i) 
            if (!Character.isLowerCase(tok.charAt(i))) 
                return true;
        return false;
    }

    public static void main(String args[]) {
	InterestingPhrases cmd = new InterestingPhrases(args);
	cmd.run();
    }

}