import org.supercsv.io.CsvListReader;
import org.supercsv.prefs.CsvPreference;
import org.supercsv.io.CsvListWriter;

import com.aliasi.util.Files;
import com.aliasi.util.Strings;

import com.aliasi.chunk.Chunk;
import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.ChunkFactory;
import com.aliasi.chunk.Chunking;
import com.aliasi.chunk.ChunkingImpl;
import com.aliasi.chunk.ChunkingEvaluation;
import com.aliasi.chunk.ChunkerEvaluator;

import com.aliasi.dict.Dictionary;
import com.aliasi.dict.DictionaryEntry;
import com.aliasi.dict.ExactDictionaryChunker;
import com.aliasi.dict.TrieDictionary;

import com.aliasi.sentences.IndoEuropeanSentenceModel;
import com.aliasi.sentences.HeuristicSentenceModel;
import com.aliasi.sentences.SentenceChunker;
import com.aliasi.sentences.SentenceModel;

import com.aliasi.hmm.HiddenMarkovModel;
import com.aliasi.hmm.HmmDecoder;

import com.aliasi.util.AbstractExternalizable;
import com.aliasi.util.FastCache;
import com.aliasi.util.ObjectToCounterMap;
import com.aliasi.util.Strings;

import com.aliasi.tag.Tagging;

import com.aliasi.tokenizer.LowerCaseTokenizerFactory;
import com.aliasi.tokenizer.Tokenizer;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.TokenNGramTokenizerFactory;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileInputStream;

import java.io.InputStreamReader;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.zip.GZIPInputStream;


public class KeyPhrases {

    static ObjectToCounterMap<String> mTokenNgramCounts = new ObjectToCounterMap<String>();
    static ObjectToCounterMap<String> mTokenNgramDocCounts = new ObjectToCounterMap<String>();

    public static List<String[]> parseFile(File file) throws IOException {
	List<String[]> returnList = new ArrayList<String[]>();
	FileInputStream fileInput = new FileInputStream(file);
	InputStreamReader fileReader 
	    = new InputStreamReader(fileInput,Strings.UTF8);
	CsvListReader reader
	    = new CsvListReader(fileReader, CsvPreference.EXCEL_PREFERENCE);
	int counter = 2;
	try {
	    String[] header = reader.getCSVHeader(true);
	    List<String> csv = null;
	    while ((csv = reader.read()) != null) {
		String id = csv.get(ID_INDEX);
		String title = csv.get(TITLE_INDEX);
		String body = csv.get(CSV_BODY_INDEX);
		//		System.out.println(body);
		String[] info = {id,title,body};
		returnList.add(info);
		++counter;
	    }
	} finally {
	    reader.close();
	}
	return returnList;
    }

    static Chunker loadDictionaryChunker(File file, TokenizerFactory tokFactory) throws IOException {
	System.out.println("Reading in Dictionary file: " + file);
	Dictionary<String> dict = new TrieDictionary<String>();
	List<String[]> data = parseFile(file);
	for (String[] csv : data) {
	    dict.addEntry(new DictionaryEntry<String>(csv[BODY_INDEX],DICT_IP_MARK));
	}
	return new ExactDictionaryChunker(dict,tokFactory);
    }

    public static Set<String>loadStopPhrases(File file) throws IOException {
	System.out.println("Reading in stop phrase file: " + file);
	Set<String> returnSet = new HashSet<String>();
	List<String[]> data = parseFile(file);
	for (String[] csv : data) {
	    if (csv.length < BODY_INDEX) {
		returnSet.add(csv[BODY_INDEX]);
	    }
	}
	return returnSet;
    }

    public static void main(String[] args) throws IOException,ClassNotFoundException {
	File inFile = new File(args[0]);
	List<String[]> docs = parseFile(inFile);
        File hmmFile = new File(args[1]);
        int cacheSize = Integer.valueOf(args[2]);
        FastCache<String,double[]> cache = new FastCache<String,double[]>(cacheSize);
	PhraseChunker npChunker = loadNpChunker(hmmFile,cache);
	Set<String> stopPhrases = loadStopPhrases(new File(args[6]));
	TokenizerFactory tokFactory 
	    = new IndoEuropeanTokenizerFactory();
	Chunker dictionaryChunker = loadDictionaryChunker(new File(args[5]),tokFactory);
	TokenizerFactory normalizedTokFactory = tokFactory;
	//	    = new LowerCaseTokenizerFactory(tokFactory);
	//loadBackgroundData(new File(args[4]),normalizedTokFactory);
	Chunker paragraphChunker = new ParagraphChunker();
	Chunker sentenceChunker = new SentenceChunker(tokFactory,
						      new JobSentenceModel());
	PhraseFilter phraseFilter = // new NoPhraseFilter();
	    new AdHocPhraseFilter(stopPhrases,
				  mTokenNgramCounts,
				  mTokenNgramDocCounts,
				  normalizedTokFactory);
	File output = new File(args[3]);
	System.out.println("Writing to: " + output);
	SentenceNpChunker sentNpChunker 
	    = new SentenceNpChunker(npChunker,dictionaryChunker,sentenceChunker,paragraphChunker,tokFactory,phraseFilter);
	if (args[7].equals("dumpKeyPhrases")) {
	    dumpChunks(docs,sentNpChunker,output);
	}
	else if (args[7].equals("annotation")) {
	    sentNpChunker.setVerboseChunking(false);//set to true to get sent and paragraph chunks
	    writeChunksInline(docs,sentNpChunker,output);
	}
	else if (args[7].equals("evaluation")) {
	    sentNpChunker.setEvaluationChunking(false);
	    sentNpChunker.setVerboseChunking(true);//set to true to get sent and paragraph chunks
	    writeDebugChunks(docs,sentNpChunker,output);
	    sentNpChunker.setVerboseChunking(false);
	    sentNpChunker.setEvaluationChunking(true);
	    evaluate(docs,sentNpChunker);
	}
	else {
	    System.out.println("No valid output method, got: " +args[7]);
	}
	    
    }
    
    static void evaluate(List<String[]> docs, Chunker chunker) {
	ChunkerEvaluator eval = new ChunkerEvaluator(chunker);
	for (String[] entry : docs) {
	    String body = entry[BODY_INDEX];
	    ChunkingImpl truthChunking = parseBody(body);
	    eval.handle(truthChunking);
	}
	System.out.println(eval);
    }
    
    static void writeDebugChunks(List<String[]> docs, Chunker chunker, File output) throws IOException {
	FileOutputStream stream =  new FileOutputStream(output);
	OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	CsvListWriter writer 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 
	ArrayList<String> row = new ArrayList<String>();
	row.add("ID");
	row.add("Title");
	row.add("Body");
	writer.write(row);
	for (String[] entry : docs) {
	    row.clear();
	    String body = entry[BODY_INDEX];
	    ChunkingImpl truthChunking = parseBody(body);
	    Chunking systemChunking 
		= chunker.chunk(truthChunking.charSequence());
	    truthChunking.addAll(systemChunking.chunkSet());
	    String results = visualizeResultsVerbose(truthChunking);
	    row.add(entry[ID_INDEX]);
	    row.add(entry[TITLE_INDEX]);
	    row.add(results);
	    writer.write(row);
	}
	writer.close();
    }

    static void writeChunksInline(List<String[]> docs, Chunker chunker, File output) throws IOException {
	System.out.println("Writing chunksinline");
	FileOutputStream stream =  new FileOutputStream(output);
	OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	CsvListWriter writer 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 
	ArrayList<String> row = new ArrayList<String>();
	row.add("ID");
	row.add("Title");
	row.add("Body");
	writer.write(row);
	for (String[] entry : docs) {
	    row.clear();
	    String body = entry[BODY_INDEX];
	    ChunkingImpl truthChunking = parseBody(body); //nothing to parse for unannotated files, but does no harm
	    Chunking systemChunking 
		= chunker.chunk(truthChunking.charSequence());
	    String results = visualizeResultsVerbose(systemChunking);
	    row.add(entry[ID_INDEX]);
	    row.add(entry[TITLE_INDEX]);
	    row.add(results);
	    writer.write(row);
	}
	writer.close();
    }

    static void dumpChunks(List<String[]> docs, Chunker chunker, File output) throws IOException {
	FileOutputStream stream =  new FileOutputStream(output);
	OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	CsvListWriter writer 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 
	ArrayList<String> row = new ArrayList<String>();
	row.add("Interesting Phrase");
	writer.write(row);
	for (String[] entry : docs) {
	    
	    String body = entry[BODY_INDEX];
	    Chunking systemChunking 
		= chunker.chunk(body);
	    for(Chunk chunk : systemChunking.chunkSet()) {
		row.clear();
		String phrase = body.substring(chunk.start(),chunk.end());
		row.add(phrase);
		writer.write(row);
	    }
	}
	writer.close();
    }
    

    static PhraseChunker loadNpChunker(File hmmFile, FastCache<String,double[]> cache) throws IOException, ClassNotFoundException {
	// read HMM for pos tagging
	HiddenMarkovModel posHmm;
	posHmm
	    = (HiddenMarkovModel)
	    AbstractExternalizable.readObject(hmmFile);
        // construct chunker
        HmmDecoder posTagger  = new HmmDecoder(posHmm,null,cache);
        TokenizerFactory tokenizerFactory = IndoEuropeanTokenizerFactory.INSTANCE;
        PhraseChunker npChunker = new PhraseChunker(posTagger,tokenizerFactory);
	return npChunker;
    }
	


    static void loadBackgroundData(File file,
				   TokenizerFactory tokFactory) throws IOException {
	//too many different formats to worry about parsing, it will get sorted out in the counts.

	int shortestPhrase = 1;
	int longestPhrase = 3;
	tokFactory 
	    = new TokenNGramTokenizerFactory(tokFactory,shortestPhrase,longestPhrase);
	process(file,tokFactory); //use for non csv formatted
	//processDoc(file,tokFactory); //use for csv formatted
    }

    static void process(File file, TokenizerFactory tokFactory) throws IOException {
	if (file.isDirectory()) {
	    for (File subFile : file.listFiles()) {
		process(subFile,tokFactory);
	    }
	}
	else {
	    System.out.println("reading background data file: " + file);
	    String[] lines = Files.readFromFile(file,Strings.UTF8).split("\n");
	    for (int i = 1; i < lines.length; ++i) {
		if (lines[i].startsWith("<")) {
		    continue;
		}
		char[] chars = lines[i].toCharArray();
		Tokenizer tok = tokFactory.tokenizer(chars,0,chars.length);
		String phrase;
		while ((phrase = tok.nextToken()) != null) {
		    //		System.out.println(phrase);
			mTokenNgramCounts.increment(phrase);
			mTokenNgramDocCounts.increment(phrase);
		}
	    }
	}
    }

    static void processDoc(File file, TokenizerFactory tokFactory) throws IOException {
	if (file.isDirectory()) {
	    for (File subFile : file.listFiles()) {
		processDoc(subFile,tokFactory);
	    }
	}
	else {
	    System.out.println("Reading in background file " + file);
	    List<String[]> lines = parseFile(file);
	    for (int i = 0; i < lines.size(); ++i) {
		char[] chars = lines.get(i)[2].toCharArray();
		Tokenizer tok = tokFactory.tokenizer(chars,0,chars.length);
		String phrase;
		Set<String> seenInDoc = new HashSet<String>();
		while ((phrase = tok.nextToken()) != null) {
		    //		System.out.println(phrase);
		    if (!seenInDoc.contains(phrase)) {
			//			System.out.println("adding:" + phrase); 
			mTokenNgramDocCounts.increment(phrase);
			seenInDoc.add(phrase);
		    }
		    mTokenNgramCounts.increment(phrase);
		}
	    }
	}
    }
	
    
    static String visualizeResultsVerbose(Chunking chunks) {
	char[] textChars = chunks.charSequence().toString().toCharArray();
	StringBuilder[] annot = new StringBuilder[textChars.length + 1];
	for (int i = 0; i < annot.length; ++i) {
	    annot[i] = new StringBuilder();
	}
	Chunk[] chunkList = chunks.chunkSet().toArray(new Chunk[0]);
	final Map<String,Integer> ordering = new HashMap<String,Integer>();
	ordering.put(TRUTH_IP_MARK,0);
	ordering.put(SYS_IP_MARK,1);
	ordering.put(SENT_MARK,2);
	ordering.put(PARA_MARK,3);

	Arrays.sort(chunkList,new Comparator<Chunk> () {
		public int compare(Chunk c1, Chunk c2) {
		    int result = c1.end() - c2.end();
		    if (result == 0) {
			return ordering.get(c1.type()) - ordering.get(c2.type());
		    }
		    return result;
		}
	    }
	    );
	for (int i = 0; i < chunkList.length; ++i) {
	    annot[chunkList[i].end()].append("</"+chunkList[i].type()+">");
	}
	Arrays.sort(chunkList,new Comparator<Chunk> () {
		public int compare(Chunk c1, Chunk c2) {
		    int result = c1.start() - c2.start();
		    if (result == 0) {
			return -(ordering.get(c1.type()) - ordering.get(c2.type()));
		    }
		    return result;
		}
	    }
	    );
	for (int i = 0; i < chunkList.length; ++i) {
	    annot[chunkList[i].start()].append("<"+chunkList[i].type()+">");
	}
	StringBuilder sb = new StringBuilder();
	for (int i = 0; i < textChars.length; ++i) {
	    sb.append(annot[i]);
	    sb.append(textChars[i]);
	}
	return sb.toString();
    }

    static int CSV_BODY_INDEX = 3;
    static int BODY_INDEX = 2;
    static int TITLE_INDEX = 1;
    static int ID_INDEX = 0;
    static String PARA_MARK = "p";
    static String SENT_MARK = "s";
    static String TRUTH_IP_MARK = "t";
    static String SYS_IP_MARK = "i";
    static String DICT_IP_MARK = "d";
    static int OPEN = 0;
    static int CLOSE = 1;

    static ChunkingImpl parseBody (String text) {
	List<Chunk> chunks = new ArrayList<Chunk>();
	StringBuilder sb = new StringBuilder();
	char[] chars = text.toCharArray();
	boolean inChunk = false;
	boolean leftBracket = false;
	int offset = 0;
	int start = 0;
	for (int i=0; i < chars.length; ++i) {
	    //System.out.print(chars[i]);
	    if (chars[i] == '<') {
		if (inChunk) {
		    throw new RuntimeException("illegal sequence, open chunk within chunk at char:" + i);
		}
		inChunk = true;
		start = i - offset;
		++offset;
	    }
	    else if (chars[i] == '>') {
		if (!inChunk) {
		    throw new RuntimeException("illegal sequence, close chunk with out open < at char:" + i);
		}
		inChunk = false;
		Chunk chunk = ChunkFactory.createChunk(start,i - offset,TRUTH_IP_MARK);
		chunks.add(chunk);
		++offset;
	    }
	    else {
		sb.append(chars[i]);
	    }
	}
	if (inChunk) {
	    throw new RuntimeException("unfinished chunk");
	}
	ChunkingImpl chunking = new ChunkingImpl(sb.toString());
	chunking.addAll(chunks);
	return chunking;
    }
}