import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;

import com.aliasi.cluster.LatentDirichletAllocation;

import com.aliasi.corpus.ObjectHandler;

import com.aliasi.symbol.MapSymbolTable;
import com.aliasi.symbol.SymbolTable;

import com.aliasi.tokenizer.EnglishStopTokenizerFactory;
import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.LowerCaseTokenizerFactory;
import com.aliasi.tokenizer.RegExFilteredTokenizerFactory;
import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.StopTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;

import com.aliasi.util.Strings;
import com.aliasi.util.AbstractExternalizable;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.Random;

public class Lda {

    static final int NUM_FOLDS = 1;
 
    public static void main(String[] args) throws IOException {
	short numTopics = Short.valueOf(args[0]);    	
        File dataDir = new File(args[1]);// + ".csv");
	String query = args[2];
        FilteredXValCorpus corpus = new FilteredXValCorpus(NUM_FOLDS,dataDir,args[3]);
	
        CharSequence[] texts = corpus.texts();
        System.out.println("corpus.size=" + texts.length);
	System.out.println("number of topics=" + numTopics);
	System.out.println("category filter=" + corpus.categorySet());
        int minTokenCount = 2;
        double topicPrior = 0.1;
        double wordPrior = 0.01;
        int burninEpochs = 0;
        int sampleLag = 1;
        int numSamples = 10000;
	long randomSeed = 6474835;

	
        TokenizerFactory tokenizerFactory = new QueryFilterTokenizerFactory(query);
	int maxCountForFilter = 3000;
	tokenizerFactory = new CommonWordFilterTokenizerFactory(tokenizerFactory,new File(args[4]),maxCountForFilter);

        MapSymbolTable symbolTable = new MapSymbolTable();

        int[][] docTokens
            = LatentDirichletAllocation
            .tokenizeDocuments(texts,tokenizerFactory,symbolTable,minTokenCount);

        LdaReportingHandler handler
            = new LdaReportingHandler(symbolTable);

        LatentDirichletAllocation.GibbsSample sample
            = LatentDirichletAllocation.gibbsSampler(docTokens,
                          numTopics,
                          topicPrior,
                          wordPrior,
                          burninEpochs,
                          sampleLag,
                          numSamples,
                          new Random(randomSeed),
                          handler);
        int maxWordsPerTopic = 20;
        int maxTopicsPerDoc = 5;
        boolean reportTokens = true;
	File labelsFile = new File("clusteringOutput",query+".labels");
        LdaReportingHandler.fullReport(sample,
                                       maxWordsPerTopic,
                                       maxTopicsPerDoc,
                                       reportTokens,
                                       symbolTable,
				       labelsFile);
    
	AbstractExternalizable.serializeTo(sample.lda(),new File("models",query + ".LatentDirichletAllocation"));
	AbstractExternalizable.serializeTo(symbolTable,new File("models",query + ".MapSymbolTable"));
    }
    

    



    
}