import com.aliasi.chunk.Chunk;
import com.aliasi.chunk.ChunkFactory;
import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.Chunking;
import com.aliasi.chunk.ChunkingImpl;

import java.util.List;

import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;

public class ParagraphChunker implements Chunker {
    
    public Chunking chunk(char[] chars, int start, int end) {
	ChunkingImpl returnChunking = new ChunkingImpl(chars,start,end);
	boolean inPara = true;
	int newlineCount = 0;
	int paraStart = start;
	for (int i = start; i < end; ++i) {
	    if (chars[i] == '\n' 
		|| chars[i] == '\r') {
		inPara = true;
		++newlineCount;
	    }
	    else {
		if (inPara && newlineCount > 1) {	
		    Chunk chunk = ChunkFactory.createChunk(paraStart,i,KeyPhrases.PARA_MARK);
		    returnChunking.add(chunk);
		    newlineCount = 0;
		    inPara = false;
		    paraStart = i;
		}
		else { //anything starts a para unless it ended one
		    inPara = true;
		    newlineCount = 0;
		}
	    }
	}
	if (inPara) {
	    returnChunking.add(ChunkFactory.createChunk(paraStart,end,KeyPhrases.PARA_MARK));
	}
	return returnChunking;
		
    }
    
    public Chunking chunk(CharSequence cseq) {
	char[] chars = cseq.toString().toCharArray();
	return chunk(chars,0,chars.length);
    }

    public static void main(String[] args) throws IOException, FileNotFoundException {
	List<String[]> docs = KeyPhrases.parseFile(new File(args[0]));
	Chunker chunker = new ParagraphChunker();
	
	for (String[] doc : docs) {
	    System.out.println(doc.length);
	    Chunking paragraphs = chunker.chunk(doc[KeyPhrases.BODY_INDEX]);
	    for (Chunk para : paragraphs.chunkSet()) {
		System.out.println("<p>" + doc[KeyPhrases.BODY_INDEX].substring(para.start(),para.end()) + "</p>");
	    }
	    
	}
    }
}