import com.aliasi.classify.JointClassifierEvaluator;

import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.classify.ConditionalClassification;
import com.aliasi.classify.JointClassification;
import com.aliasi.classify.ConfusionMatrix;
import com.aliasi.classify.DynamicLMClassifier;
import com.aliasi.classify.LogisticRegressionClassifier;
import com.aliasi.corpus.ObjectHandler;

import com.aliasi.io.Reporter;
import com.aliasi.io.Reporters;

import com.aliasi.lm.NGramProcessLM;

import com.aliasi.stats.AnnealingSchedule;
import com.aliasi.stats.RegressionPrior;

import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.TokenFeatureExtractor;

import com.aliasi.util.AbstractCommand;
import com.aliasi.util.FeatureExtractor;
import com.aliasi.util.ObjectToDoubleMap;
import com.aliasi.util.Pair;
import com.aliasi.util.Strings;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;


import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import java.util.Set;
import java.util.HashSet;


public class PartialXvalLM extends AbstractCommand {

    int mNumFolds = 10;
    int mMaxNgram = 5;
    final ObjectToDoubleMap<Pair<String,AnnotatedConditionalClassification>> mObjectToDoubleMap 
	= new ObjectToDoubleMap<Pair<String,AnnotatedConditionalClassification>> ();

    public PartialXvalLM(String[] args) {
	super(args);
    }

    public void run() {
	try {

	    FilteredXValCorpus corpus 
		= new FilteredXValCorpus(mNumFolds, getArgumentFile("trainingData"),getArgument("categories"));
	    process(corpus);

	    Set<String> keepersTest = new HashSet<String>();
	    keepersTest.add("");
	    //mNumFolds = 1;//visit Test visits whole corpus
	    FilteredXValCorpus corpusUnknown 
		= new FilteredXValCorpus(mNumFolds, getArgumentFile("trainingData"),keepersTest);
	    
	    processUnAnnotated(corpus,corpusUnknown);

	    File outDir = getArgumentFile("outdir");
	    String outFile = getArgument("outfile");
	    TweetWriter.writeCsv(mObjectToDoubleMap, outDir, outFile,"");
	}
	catch (Exception e) {
	    System.out.println("Problem with program. Got exception " + e);
            e.printStackTrace(System.out);
	}
    }

    void processUnAnnotated(FilteredXValCorpus corpusTrain,FilteredXValCorpus corpusTest) throws IOException {

	PrintWriter progressWriter = new PrintWriter(System.out,true);

        progressWriter.println("Reading data.");

	String[] categories 
	    = corpusTrain.categorySet().toArray(Strings.EMPTY_STRING_ARRAY);

	final DynamicLMClassifier<NGramProcessLM> classifier 
		= DynamicLMClassifier.createNGramProcess(categories,mMaxNgram);
	    corpusTrain.visitCorpus(classifier);

	Reporter reporter = Reporters.writer(progressWriter);
            
	corpusTest.visitCorpus(new ObjectHandler<Classified<CharSequence>> (){
		public void handle(Classified<CharSequence> c) {
		    String tweet = c.getObject().toString();
		    final ConditionalClassification classification 
			= classifier.classify(tweet);
		    String truthClassification = c.getClassification().bestCategory();
		    AnnotatedConditionalClassification annotatedClassification
			= new AnnotatedConditionalClassification(classification,truthClassification);
		    Pair<String,AnnotatedConditionalClassification> pair 
			= new Pair<String,AnnotatedConditionalClassification>(tweet,
									      annotatedClassification);
		    mObjectToDoubleMap.set(pair,
					   classification.score(0));
		    //    System.out.println(tweet);
		    //System.out.println(classification);
		}
	    });
    }

    void process(FilteredXValCorpus corpus) throws IOException {

	PrintWriter progressWriter = new PrintWriter(System.out,true);

        progressWriter.println("Reading data for proceess");
	
	corpus.permuteCorpus(new Random(7117)); // destroys runs of categories

        boolean storeInputs = true;
	String[] categories 
	    = corpus.categorySet().toArray(Strings.EMPTY_STRING_ARRAY);
	JointClassifierEvaluator<CharSequence> evaluator 
	    = new JointClassifierEvaluator<CharSequence>(null,
							 corpus.categorySet().toArray(Strings.EMPTY_STRING_ARRAY),
							 storeInputs);
	
	System.out.println("Number of folds: " + corpus.numFolds());
	
        for (int fold = 0; fold < mNumFolds; ++fold) {
            corpus.setFold(fold);
	    System.out.println("Fold: " + fold);
	    DynamicLMClassifier<NGramProcessLM> classifier 
		= DynamicLMClassifier.createNGramProcess(categories,mMaxNgram);
	    corpus.visitTrain(classifier);
	    evaluator.setClassifier(classifier);
	    corpus.visitTest(evaluator);
        }

	progressWriter.println(evaluator.confusionMatrix());

	collectXvalCases(evaluator);
    }

    void collectXvalCases(JointClassifierEvaluator<CharSequence> evaluator) {
	for (String cat : evaluator.categories()) {
	    List<Classified<CharSequence>> truePos 
		= evaluator.truePositives(cat);
	    for (Classified<CharSequence> ccs : truePos) {
		addToMap(ccs, cat);
	    }
	    List<Classified<CharSequence>> falseNeg 
		= evaluator.falseNegatives(cat);
	    for (Classified<CharSequence> ccs : falseNeg) {
		addToMap(ccs, cat);
	    }
	}
    }
    
    void addToMap(Classified<CharSequence> ccs, String cat) {
	String tweet = ccs.getObject().toString();
	ConditionalClassification responseClassification 
	    = (ConditionalClassification) ccs.getClassification();
	int rank = 0;
	String responseCategory 
	    = responseClassification.bestCategory();
	AnnotatedConditionalClassification annotClassification 
	    = new AnnotatedConditionalClassification(responseClassification,
					  cat);
	Pair<String,AnnotatedConditionalClassification> pair 
	    = new Pair<String,AnnotatedConditionalClassification>(tweet,annotClassification);
	double score = responseClassification.score(rank);
	mObjectToDoubleMap.set(pair,score);
    }	    

    
    public static void main(String[] args) {
	PartialXvalLM cmd = new PartialXvalLM(args);
	cmd.run();
    }

}