import com.aliasi.classify.ConditionalClassifierEvaluator;

import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.classify.ConditionalClassification;
import com.aliasi.classify.ConfusionMatrix;
import com.aliasi.classify.LogisticRegressionClassifier;
import com.aliasi.corpus.ObjectHandler;

import com.aliasi.io.Reporter;
import com.aliasi.io.Reporters;

import com.aliasi.stats.AnnealingSchedule;
import com.aliasi.stats.RegressionPrior;

import com.aliasi.tokenizer.NGramTokenizerFactory;
import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.TokenFeatureExtractor;

import com.aliasi.util.AbstractCommand;
import com.aliasi.util.FeatureExtractor;
import com.aliasi.util.ObjectToDoubleMap;
import com.aliasi.util.Pair;
import com.aliasi.util.Strings;
import com.aliasi.io.LogLevel;


import java.util.ArrayList;
import java.util.List;
import java.util.Random;


import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import java.util.Set;
import java.util.HashSet;


public class PartialXvalLogReg extends AbstractCommand {

    int mNumFolds = 10;

    final ObjectToDoubleMap<Pair<String,AnnotatedConditionalClassification>> mObjectToDoubleMap 
	= new ObjectToDoubleMap<Pair<String,AnnotatedConditionalClassification>> ();

    public PartialXvalLogReg(String[] args) {
	super(args);
    }

    public void run() {
	try {

	    FilteredXValCorpus corpus 
		= new FilteredXValCorpus(mNumFolds, getArgumentFile("trainingData"),getArgument("categories"));
	    System.out.println(corpus);
	    
	    process(corpus);
	    
	    Set<String> keepersTest = new HashSet<String>();
	    keepersTest.add("");
	    mNumFolds = 0;
	    FilteredXValCorpus corpusUnknown 
		= new FilteredXValCorpus(mNumFolds, getArgumentFile("trainingData"),keepersTest);
	    
	    processUnAnnotated(corpus,corpusUnknown);

	    File outDir = getArgumentFile("outdir");
	    String outFile = getArgument("outfile");
	    TweetWriter.writeCsv(mObjectToDoubleMap, outDir, outFile,"");
	    
	}
	catch (Exception e) {
	    System.out.println("Problem with program. Got exception " + e);
            e.printStackTrace(System.out);
	}
    }

    void processUnAnnotated(FilteredXValCorpus corpusTrain,FilteredXValCorpus corpusTest) throws IOException {

	PrintWriter progressWriter = new PrintWriter(System.out,true);

        progressWriter.println("Reading data.");
	
	TokenizerFactory tokenizerFactory
	               = new RegExTokenizerFactory("\\p{L}+|\\d+"); // letter+ | digit+
	//= new NGramTokenizerFactory(3,5);

	FeatureExtractor<CharSequence> featureExtractor
	    = new TokenFeatureExtractor(tokenizerFactory);
	int minFeatureCount = 2;
	boolean addInterceptFeature = false;
	boolean noninformativeIntercept = true;
	double priorVariance = 0.5;
	RegressionPrior prior 
	    = RegressionPrior.laplace(priorVariance,noninformativeIntercept);
	AnnealingSchedule annealingSchedule
	    = AnnealingSchedule.exponential(0.002,0.9975);
	double minImprovement = 0.0000001;
	int minEpochs = 100;
	int maxEpochs = 1000;

	int numFolds = 1;
	corpusTrain.setFold(numFolds);
	System.out.println("Number of folds: " + numFolds);
	Reporter reporter = Reporters.writer(progressWriter);
	final LogisticRegressionClassifier<CharSequence> classifier
	    = LogisticRegressionClassifier.train(corpusTrain,
						 featureExtractor,
						 minFeatureCount,
						 addInterceptFeature,
						 prior,
						 annealingSchedule,
						 minImprovement,
						 minEpochs,
						 maxEpochs,
						 reporter);
            
	System.out.println(classifier); 
	corpusTest.visitCorpus(new ObjectHandler<Classified<CharSequence>> (){
		public void handle(Classified<CharSequence> c) {
		    String tweet = c.getObject().toString();
		    final ConditionalClassification classification 
			= classifier.classify(tweet);
		    String truthClassification = c.getClassification().bestCategory();
		    AnnotatedConditionalClassification annotatedClassification
			= new AnnotatedConditionalClassification(classification,truthClassification);
		    Pair<String,AnnotatedConditionalClassification> pair 
			= new Pair<String,AnnotatedConditionalClassification>(tweet,
									      annotatedClassification);
		    mObjectToDoubleMap.set(pair,
					   classification.conditionalProbability(0));
		    //    System.out.println(tweet);
		    //System.out.println(classification);
		}
	    });
    }


    void process(FilteredXValCorpus corpus) throws IOException {

	PrintWriter progressWriter = new PrintWriter(System.out,true);

        progressWriter.println("Reading data for proceess");
	
	corpus.permuteCorpus(new Random(7117)); // destroys runs of categories

	TokenizerFactory tokenizerFactory
	    = new NGramTokenizerFactory(3,5);
	    //= new RegExTokenizerFactory("\\p{L}+|\\d+"); // letter+ | digit+
	FeatureExtractor<CharSequence> featureExtractor
	    = new TokenFeatureExtractor(tokenizerFactory);
	int minFeatureCount = 2;
	boolean addInterceptFeature = false;
	boolean noninformativeIntercept = true;
	double priorVariance = 0.5;
	RegressionPrior prior 
	    = RegressionPrior.laplace(priorVariance,noninformativeIntercept);
	AnnealingSchedule annealingSchedule
	    = AnnealingSchedule.exponential(0.002,0.9975);
	double minImprovement = 0.0000001;
	int minEpochs = 100;
	int maxEpochs = 10000;

        boolean storeInputs = true;
	ConditionalClassifierEvaluator<CharSequence> evaluator 
	    = new ConditionalClassifierEvaluator<CharSequence>(null,
                                                               corpus.categorySet().toArray(Strings.EMPTY_STRING_ARRAY),
                                                               storeInputs);

	System.out.println("Number of folds: " + corpus.numFolds());
        for (int fold = 0; fold < mNumFolds; ++fold) {
            corpus.setFold(fold);
	    System.out.println("Fold: " + fold);
            Reporter reporter = Reporters.writer(progressWriter);
	    reporter.setLevel(LogLevel.INFO);
            LogisticRegressionClassifier<CharSequence> classifier
                = LogisticRegressionClassifier.train(corpus,
                                                     featureExtractor,
                                                     minFeatureCount,
                                                     addInterceptFeature,
                                                     prior,
                                                     annealingSchedule,
                                                     minImprovement,
                                                     minEpochs,
                                                     maxEpochs,
                                                     reporter);
	    evaluator.setClassifier(classifier);
	    corpus.visitTest(evaluator);
        }

	progressWriter.println(evaluator.confusionMatrix());

	collectXvalCases(evaluator);
    }

    void collectXvalCases(ConditionalClassifierEvaluator<CharSequence> evaluator) {
	for (String cat : evaluator.categories()) {
	    List<Classified<CharSequence>> truePos 
		= evaluator.truePositives(cat);
	    for (Classified<CharSequence> ccs : truePos) {
		addToMap(ccs, cat);
	    }
	    List<Classified<CharSequence>> falseNeg 
		= evaluator.falseNegatives(cat);
	    for (Classified<CharSequence> ccs : falseNeg) {
		addToMap(ccs, cat);
	    }
	}
    }
    
    void addToMap(Classified<CharSequence> ccs, String cat) {
	String tweet = ccs.getObject().toString();
	ConditionalClassification responseClassification 
	    = (ConditionalClassification) ccs.getClassification();
	int rank = 0;
	String responseCategory 
	    = responseClassification.bestCategory();
	AnnotatedConditionalClassification annotClassification 
	    = new AnnotatedConditionalClassification(responseClassification,
					  cat);
	Pair<String,AnnotatedConditionalClassification> pair 
	    = new Pair<String,AnnotatedConditionalClassification>(tweet,annotClassification);
	double score = responseClassification.conditionalProbability(rank);
	mObjectToDoubleMap.set(pair,score);
    }	    

    
    public static void main(String[] args) {
	PartialXvalLogReg cmd = new PartialXvalLogReg(args);
	cmd.run();
    }

}