import com.aliasi.classify.ConditionalClassifierEvaluator;

import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.classify.ConditionalClassification;
import com.aliasi.classify.ConfusionMatrix;
import com.aliasi.classify.TradNaiveBayesClassifier;

import com.aliasi.corpus.ObjectHandler;

import com.aliasi.io.Reporter;
import com.aliasi.io.Reporters;

import com.aliasi.stats.AnnealingSchedule;
import com.aliasi.stats.RegressionPrior;

import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.NGramTokenizerFactory;
import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.RegExFilteredTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.TokenFeatureExtractor;

import com.aliasi.util.AbstractCommand;
import com.aliasi.util.FeatureExtractor;
import com.aliasi.util.ObjectToDoubleMap;
import com.aliasi.util.Pair;
import com.aliasi.util.Strings;
import com.aliasi.io.LogLevel;


import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import java.util.Set;
import java.util.HashSet;


public class PartialXvalNaiveBayes extends AbstractCommand {

    int mNumFolds = 10;
    TokenizerFactory mTokenizerFactory;
    double mTokenPrior;
    double mCategoryPrior;
    double mLengthNorm;

    final ObjectToDoubleMap<Pair<String,AnnotatedConditionalClassification>> mObjectToDoubleMap 
	= new ObjectToDoubleMap<Pair<String,AnnotatedConditionalClassification>> ();

    public PartialXvalNaiveBayes(String[] args) {
	super(args);	
    }

    public void run() {
	try {
	    mTokenizerFactory
		= new NGramTokenizerFactory(2,4);
		//= IndoEuropeanTokenizerFactory.INSTANCE;
	    
	    //	    mTokenizerFactory
	    //= new RegExNonMatchingTokenizerFactory(mTokenizerFactory,getArgument("filterRegex"));
	    
	    FilteredXValCorpus corpus 
		= new FilteredXValCorpus(mNumFolds, getArgumentFile("trainingData"),getArgument("categories"));
	    mTokenPrior = getArgumentDouble("tokenPrior");
	    mCategoryPrior = getArgumentDouble("categoryPrior");
	    mLengthNorm = getArgumentDouble("lengthNorm");
	    //	    System.out.println(corpus);
	    
	    process(corpus);
	    
	    Set<String> keepersTest = new HashSet<String>();
	    keepersTest.add("");
	    mNumFolds = 0;
	    FilteredXValCorpus corpusUnknown 
		= new FilteredXValCorpus(mNumFolds, getArgumentFile("trainingData"),keepersTest);

	    processUnAnnotated(corpus,corpusUnknown);
	    File outDir = getArgumentFile("outdir");
	    String outFile = getArgument("outfile");
	    TweetWriter.writeCsv(mObjectToDoubleMap, outDir, outFile,"");
	    
	}
	catch (Exception e) {
	    System.out.println("Problem with program. Got exception " + e);
            e.printStackTrace(System.out);
	}
    }


void processUnAnnotated(FilteredXValCorpus corpusTrain, FilteredXValCorpus corpusTest) throws IOException {

	FeatureExtractor<CharSequence> featureExtractor
	    = new TokenFeatureExtractor(mTokenizerFactory);
	int numFolds = 1;

	corpusTrain.setFold(numFolds);

	System.out.println("Number of folds: " + numFolds);

	
	final TradNaiveBayesClassifier classifier 
	    = new TradNaiveBayesClassifier(corpusTrain.categorySet(),mTokenizerFactory,
					   mCategoryPrior,
					   mTokenPrior,
					   mLengthNorm);

	corpusTrain.visitCorpus(classifier);
	
	corpusTest.visitCorpus(new ObjectHandler<Classified<CharSequence>> (){

		public void handle(Classified<CharSequence> c) {
		    String tweet = c.getObject().toString();
		    final ConditionalClassification classification 
			= classifier.classify(tweet);
		    String truthClassification = c.getClassification().bestCategory();
		    AnnotatedConditionalClassification annotatedClassification
			= new AnnotatedConditionalClassification(classification,truthClassification);
		    Pair<String,AnnotatedConditionalClassification> pair 
			= new Pair<String,AnnotatedConditionalClassification>(tweet,
									      annotatedClassification);
		    mObjectToDoubleMap.set(pair,
					   classification.conditionalProbability(0));
		    //    System.out.println(tweet);
		    //System.out.println(classification);
		}
	    });
    }

    void process(FilteredXValCorpus corpus) throws IOException {

	corpus.permuteCorpus(new Random(7117)); // destroys runs of categories

	String[] categoryArray 
	    = corpus.categorySet().toArray(Strings.EMPTY_STRING_ARRAY);

	Set<String> categorySet 
	    = corpus.categorySet();


        boolean storeInputs = true;
	ConditionalClassifierEvaluator<CharSequence> evaluator 
	    = new ConditionalClassifierEvaluator<CharSequence>(null,
                                                               categoryArray,
                                                               storeInputs);
	
	System.out.println("Number of folds: " + corpus.numFolds());
        for (int fold = 0; fold < mNumFolds; ++fold) {
	    TradNaiveBayesClassifier classifier 
		= new TradNaiveBayesClassifier(categorySet,mTokenizerFactory,
					       mCategoryPrior,
					       mTokenPrior,
					       mLengthNorm);
            corpus.setFold(fold);
	    System.out.println("Fold: " + fold);
	    corpus.visitTrain(classifier);
	    evaluator.setClassifier(classifier);
	    corpus.visitTest(evaluator);
        }
	System.out.println(evaluator.confusionMatrix());
	collectXvalCases(evaluator);
    }

    void collectXvalCases(ConditionalClassifierEvaluator<CharSequence> evaluator) {
	for (String cat : evaluator.categories()) {
	    List<Classified<CharSequence>> truePos 
		= evaluator.truePositives(cat);
	    for (Classified<CharSequence> ccs : truePos) {
		addToMap(ccs, cat);
	    }
	    List<Classified<CharSequence>> falseNeg 
		= evaluator.falseNegatives(cat);
	    for (Classified<CharSequence> ccs : falseNeg) {
		addToMap(ccs, cat);
	    }
	}
    }
    
    void addToMap(Classified<CharSequence> ccs, String cat) {
	String tweet = ccs.getObject().toString();
	ConditionalClassification responseClassification 
	    = (ConditionalClassification) ccs.getClassification();
	int rank = 0;
	String responseCategory 
	    = responseClassification.bestCategory();
	AnnotatedConditionalClassification annotClassification 
	    = new AnnotatedConditionalClassification(responseClassification,
					  cat);
	Pair<String,AnnotatedConditionalClassification> pair 
	    = new Pair<String,AnnotatedConditionalClassification>(tweet,annotClassification);
	double score = responseClassification.conditionalProbability(rank);
	mObjectToDoubleMap.set(pair,score);
    }	    

    
    public static void main(String[] args) {
	PartialXvalNaiveBayes cmd = new PartialXvalNaiveBayes(args);
	cmd.run();
    }

}