import org.supercsv.io.CsvListWriter;
import org.supercsv.prefs.CsvPreference;

import com.aliasi.classify.Classified;

import com.aliasi.corpus.Corpus;
import com.aliasi.corpus.ObjectHandler;
import com.aliasi.corpus.XValidatingObjectCorpus;


import com.aliasi.util.Files;
import com.aliasi.util.Streams;
import com.aliasi.util.Strings;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.HashSet;
import java.util.Set;


public class RegexAnnotateCsv {
    static int mCounter = 0;
    static int mRandomNegativesCounter = 0;
    static boolean mRandomNegatives;
    static Pattern mPattern;
    static String mPatternCategory;
    static CsvListWriter mWriter;

    static void process(File input, File output, String regex, String category, String randomNegatives) throws IOException {

	if (randomNegatives.equals("true")) {
	    mRandomNegatives = true;
	    if (category.equals(NEGATIVE_CATEGORY)) {
		throw new RuntimeException("cannot use category label '"+category+"' because it is the negative category label");
	    }
	}
	else if (randomNegatives.equals("false")) {
	    mRandomNegatives = false;
	}
	else {
	    throw new RuntimeException("must specify whether to add random negatives");
	}
	mPattern = Pattern.compile(regex);
	if (category.equals("")) {
	    throw new RuntimeException(category + " cannot be the category lable");
	}
	mPatternCategory = category;
	HashSet<String> keepers = null;
	int numFolds = 1;
	XValidatingObjectCorpus<Classified<CharSequence>> corpus 
	    = new FilteredXValCorpus(numFolds,input,keepers);
	System.out.println("filtering: " + output);
	final FileOutputStream stream =  new FileOutputStream(output);
	final OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	mWriter 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 

	ArrayList<String> row 
	    = new ArrayList<String>();
	row.add("Estimate");
	row.add("Guessed Class");
	row.add("True Class");
	row.add("Text");
	mWriter.write(row);
	filterLines(corpus);
	mWriter.close();
    }

    static void filterLines(XValidatingObjectCorpus<Classified<CharSequence>> corpus) throws IOException {

	corpus.visitCorpus(new ObjectHandler <Classified<CharSequence>>() {
		public void handle(Classified<CharSequence> c) {
		    ArrayList<String> row = new ArrayList<String>();
		    row.add("");
		    row.add("");
		    CharSequence text = c.getObject();
		    Matcher matcher = mPattern.matcher(text);
		    if (matcher.find()) {
			System.out.println(text);
			row.add(mPatternCategory);
			++mCounter;
		    }
		    else {
			String category = c.getClassification().bestCategory();
			if (mRandomNegatives 
			    && mRandomNegativesCounter < mCounter
			    && category.equals("")) {
			    row.add(NEGATIVE_CATEGORY);
			    ++mRandomNegativesCounter;
			}
			else {
			    row.add(category);
			}
		    }
		    row.add(text.toString());
		    try {
			mWriter.write(row);
	
		    }
		    catch (Exception e) {
			throw new RuntimeException(e);
		    }
		}
	    });
    }
    
    static String NEGATIVE_CATEGORY = "x";
    
    public static void main(String[] args) throws IOException {
	for (int i = 0; i < args.length; ++i) {
	    System.out.println(i + " " + args[i]);
	}
	process(new File(args[0]), new File(args[1]), args[4], args[3],args[2]);
	System.out.println(mCounter + " texts processed");
	System.out.println(mRandomNegativesCounter + " non-matching negatives added");
    }

}

