import com.aliasi.chunk.Chunk;
import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.ChunkFactory;
import com.aliasi.chunk.Chunking;
import com.aliasi.chunk.ChunkingImpl;
import com.aliasi.chunk.ChunkingEvaluation;
import com.aliasi.sentences.SentenceChunker;

import com.aliasi.util.Strings;
import com.aliasi.util.ObjectToCounterMap;

import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.Tokenizer;
import com.aliasi.tokenizer.Tokenization;

import java.util.Map;
import java.util.Set;

public class SentenceNpChunker implements Chunker {

    Chunker mPhraseChunker;
    Chunker mSentenceChunker;
    Chunker mParaChunker;
    TokenizerFactory mTokFactory;
    boolean mEvaluation = false;
    boolean mVerboseChunking = false;
    Chunker mDictChunker;
    PhraseFilter mPhraseFilter;

    public SentenceNpChunker(Chunker npChunker,  
			     Chunker dictionaryChunker,
			     Chunker sentenceChunker, 
			     Chunker paraChunker, 
			     TokenizerFactory tokFactory,
			     PhraseFilter filter) {
	mPhraseChunker = npChunker;
	mSentenceChunker = sentenceChunker;
	mParaChunker = paraChunker;
	mTokFactory = tokFactory;
	mDictChunker = dictionaryChunker;
	mPhraseFilter = filter;
    }

    public void setEvaluationChunking(boolean value) {
	mEvaluation = value;
    }

    public void setVerboseChunking(boolean value) {
	mVerboseChunking = value;
    }

    public Chunking chunk(CharSequence cSeq) {
	char[] cs = Strings.toCharArray(cSeq);
	return chunk(cs,0,cs.length);
    }

    public ChunkingImpl chunk(char[] cs, int start, int end) {
	ChunkingImpl returnChunking = new ChunkingImpl(cs,start,end);
	Chunking paragraphChunking = mParaChunker.chunk(cs,start,end);
	CharSequence bodyCs = paragraphChunking.charSequence();
	int counter = 0;
	for (Chunk paraChunk : paragraphChunking.chunkSet()) {
	    int paraOffset = paraChunk.start();
	    if (mVerboseChunking) {
		returnChunking.add(paraChunk);
	    }
	    Chunking sentChunking = mSentenceChunker.chunk(cs,paraOffset,paraChunk.end());
	    for (Chunk sentChunk : sentChunking.chunkSet()) {
		Chunk sentChunkDocOffsets 
		    = ChunkFactory.createChunk(paraOffset + sentChunk.start(),
					       paraOffset + sentChunk.end(),
					       KeyPhrases.SENT_MARK);
		if (mVerboseChunking) {
		    returnChunking.add(sentChunkDocOffsets);
		}
		int sentOffset = sentChunk.start();
		CharSequence sentence 
		    = bodyCs.subSequence(paraOffset+sentOffset,paraOffset+sentChunk.end());
		//System.out.println(sentence);
		Chunking npChunking = mPhraseChunker.chunk(sentence);
		Chunking dictChunking = mDictChunker.chunk(sentence);
		Chunking mergedChunking = ChunkingImpl.merge(dictChunking,npChunking); 
		//npChunks that overlap with dictChunks will be eliminated 
		for (Chunk npChunk : mergedChunking.chunkSet()) {
		    String phrase = sentence.subSequence(npChunk.start(),npChunk.end()).toString();

		    if (npChunk.type() == KeyPhrases.DICT_IP_MARK) {
			System.out.println("dict match with :" + phrase);
		    }
		    else if (mPhraseFilter.failPhrase(npChunk,phrase)) {
			continue;
		    }
		    System.out.println("adding :" + phrase);
		    String type = mEvaluation ? KeyPhrases.TRUTH_IP_MARK : KeyPhrases.SYS_IP_MARK;
		    Chunk nounChunk = ChunkFactory.createChunk(paraOffset + sentOffset + npChunk.start(),
							       paraOffset + sentOffset + npChunk.end(),
							       type);
		    returnChunking.add(nounChunk);
		}
	    }
	}
	return returnChunking;
    }
}