import com.aliasi.classify.ConditionalClassifierEvaluator;

import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.classify.ConditionalClassification;
import com.aliasi.classify.ConfusionMatrix;
import com.aliasi.classify.LogisticRegressionClassifier;
import com.aliasi.io.LogLevel;
import com.aliasi.io.Reporter;
import com.aliasi.io.Reporters;


import com.aliasi.stats.AnnealingSchedule;
import com.aliasi.stats.RegressionPrior;

import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.tokenizer.TokenFeatureExtractor;

import com.aliasi.util.AbstractCommand;
import com.aliasi.util.AbstractExternalizable;
import com.aliasi.util.FeatureExtractor;
import com.aliasi.util.Pair;
import com.aliasi.util.Strings;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;


import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;


public class TrainLogisticRegression extends AbstractCommand {

    int mNumFolds = 0;//entire corpus is visited on training

    public TrainLogisticRegression(String[] args) {
	super(args);
    }

    public void run() {
	try {

	    Set<String> keepers = new HashSet<String>();
	    keepers.add("x");
	    keepers.add("m");
	    FilteredXValCorpus corpus 
		= new FilteredXValCorpus(mNumFolds, getArgumentFile("trainingData"),keepers);

	    process(corpus,getArgumentFile("model"));

	}
	catch (Exception e) {
	    System.out.println("Problem with program. Got exception " + e);
            e.printStackTrace(System.out);
	}
    }

    void process(FilteredXValCorpus corpus, File model) throws IOException {

	PrintWriter progressWriter = new PrintWriter(System.out,true);

        progressWriter.println("Reading data.");

	corpus.permuteCorpus(new Random(7117)); // destroys runs of categories

	TokenizerFactory tokenizerFactory
	    = new RegExTokenizerFactory("\\p{L}+|\\d+"); // letter+ | digit+
	FeatureExtractor<CharSequence> featureExtractor
	    = new TokenFeatureExtractor(tokenizerFactory);
	int minFeatureCount = 2;
	boolean addInterceptFeature = true;
	boolean noninformativeIntercept = true;
	double priorVariance = 0.5;
	RegressionPrior prior 
	    = RegressionPrior.laplace(priorVariance,noninformativeIntercept);
	AnnealingSchedule annealingSchedule
	    = AnnealingSchedule.exponential(0.002,0.9975);
	double minImprovement = 0.0000001;
	int minEpochs = 100;
	int maxEpochs = 10000;

	Reporter reporter = Reporters.writer(progressWriter);
	reporter.setLevel(LogLevel.INFO);
	LogisticRegressionClassifier<CharSequence> classifier
	    = LogisticRegressionClassifier.train(corpus,
						 featureExtractor,
						 minFeatureCount,
						 addInterceptFeature,
						 prior,
						 annealingSchedule,
						 minImprovement,
						 minEpochs,
						 maxEpochs,
						 reporter);
            
	AbstractExternalizable.compileTo(classifier,model);
	
    }
    
    public static void main(String[] args) {
	TrainLogisticRegression cmd = new TrainLogisticRegression(args);
	cmd.run();
    }

}