import com.aliasi.classify.ConditionalClassification;

import com.aliasi.util.Files;
import com.aliasi.util.ObjectToDoubleMap;               
import com.aliasi.util.Pair;
import com.aliasi.util.ScoredObject;
import com.aliasi.util.Streams;
import com.aliasi.util.Strings;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.OutputStream;
import java.io.Writer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.supercsv.io.CsvListWriter;
import org.supercsv.prefs.CsvPreference;

import twitter4j.Tweet;

public class TweetWriter {
    
    static void writeCsv(ObjectToDoubleMap<Pair<String,AnnotatedConditionalClassification>> tweets, 
			 File dir, String filename,String suffix)
	throws IOException {
	File output = new File(dir,filename); //,suffix);
	System.out.println("Writing to: " + output.toString());
	FileOutputStream stream =  new FileOutputStream(output);
	OutputStreamWriter streamWriter 
	    = new OutputStreamWriter(stream,Strings.UTF8);
	CsvListWriter writer 
	    = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 

	ArrayList<String> row = new ArrayList<String>();
	row.add("Estimate");
	row.add("Guessed Class");
	row.add("True Class");
	row.add("Text");
	writer.write(row);

	for (ScoredObject<Pair<String,AnnotatedConditionalClassification>> sco 
		 : tweets.scoredObjectsOrderedByValueList()) {
	    Pair<String,AnnotatedConditionalClassification> classifiedTweet = sco.getObject();
	    String tweet = classifiedTweet.a();
	    AnnotatedConditionalClassification annotatedClassification = classifiedTweet.b();
	    ConditionalClassification classification = annotatedClassification.condClassification();
	    String responseCategory = classification.bestCategory();
	    String referenceCategory = annotatedClassification.truthCategory();
	    Double score = classification.conditionalProbability(0);
	    //String scoreFormatted = printf("%f4.2",score);
	    row.clear();
	    row.add(score.toString());
	    row.add(responseCategory);
	    row.add(referenceCategory);
	    row.add(tweet);
	    writer.write(row);
	}
	writer.close();
    }



    public static void writeCsv(List<Tweet> tweets, File dir, 
                                String filename, String suffix)
        throws IOException {

        File output = nextFile(dir,filename,suffix);

        OutputStream fileOut = null;
        Writer streamWriter = null;
        CsvListWriter writer = null;
        try {
            fileOut =  new FileOutputStream(output);
            streamWriter = new OutputStreamWriter(fileOut,Strings.UTF8);
            writer = new CsvListWriter(streamWriter,CsvPreference.EXCEL_PREFERENCE); 

            List<String> headerRow 
                = Arrays.asList("Estimate", "Guessed Class", 
                                "True Class", "Text");
            writer.write(headerRow);

            List<String> row = new ArrayList<String>();
            for (Tweet tweet : tweets) {
                row.clear();
                row.add("");
                row.add("");
                row.add("");
                String whitespaceNormedTweet 
		    = tweet.getText().replaceAll("\\s+"," ").trim();
                row.add(whitespaceNormedTweet);
                writer.write(row);
            }
        } finally {
            try {
                if (writer != null)
                    writer.close();
            } finally {
                try { 
                    Streams.closeQuietly(streamWriter);
                } finally {
                    Streams.closeQuietly(fileOut);
                }
            }
        }
    }

    public static final int SCORE_OFFSET = 0; 
    public static final int CLASS_OFFSET = 1; 
    public static final int ANNOTATED_CLASS_OFFSET = 2; 
    public static final int TWEET_OFFSET = 3; 

    public static File nextFile(File dir, String name, String extension) {
        File f = new File(dir,name+"."+extension);
        for (int i = 1; f.exists(); ++i)
            f  = new File(dir, name + "_" + i + "." + extension);
        return f;
    }

}